'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020, 2022
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['original/dashboard-core/js/dashboard/layout/views/Tab', 'underscore', 'jquery', 'dashboard-core/js/lib/@waca/dashboard-common/dist/utils/TextEditorConstants'], function (Tab, _, $, TextEditorConstants) {
	var TabWidget = function (_Tab) {
		_inherits(TabWidget, _Tab);

		function TabWidget() {
			_classCallCheck(this, TabWidget);

			return _possibleConstructorReturn(this, _Tab.apply(this, arguments));
		}

		TabWidget.prototype._initializeTabContent = function _initializeTabContent() {
			if (!this.model.getParent()) {
				var tabContents = this.canvas.getContent().getChildren();
				var selected = this.canvas.getSelectedContentList({ type: tabContents[0].getType() });
				return selected.length ? selected[0] : tabContents[0];
			} else {
				//tab widget..
				return this.content.getChildren()[0];
			}
		};

		TabWidget.prototype._registerEvents = function _registerEvents() {
			_Tab.prototype._registerEvents.call(this);
			this.model.on('change:showDropDown', this._updateProperties, this);
			this.model.on('change:tabFontFamily', this._updateFontFamily, this);
			this.model.on('change:tabFontFamilySubClass', this._updateFontFamily, this);
		};

		TabWidget.prototype._unregisterEvents = function _unregisterEvents() {
			_Tab.prototype._unregisterEvents.call(this);
			this.model.off('change:showDropDown', this._updateProperties, this);
			this.model.off('change:tabFontFamily', this._updateFontFamily, this);
			this.model.off('change:tabFontFamilySubClass', this._updateFontFamily, this);
		};

		TabWidget.prototype.onSelection = function onSelection(event) {
			var _this2 = this;

			var idList = event.info.value;
			if (event.name === 'change:selections:select' && !idList.find(function (id) {
				return _this2.model.id === id || _this2.model.items.find(function (item) {
					return item.getId() === id;
				});
			})) {
				return;
			}

			event.info.value.forEach(function (id) {
				var canvas = _this2.canvas;
				var content = canvas.getContent(id);
				if (content && content.getType() === 'page') {
					if (_this2.selectedModel && _this2.selectedModel.id !== id) {
						canvas.deselectContent([_this2.selectedModel.id]);
					}
					_this2._TabsWidget && _this2._TabsWidget.setState({ selectedTab: _this2._getPanelId(id) });
				}
			});
		};

		TabWidget.prototype._getFontFamilyStyles = function _getFontFamilyStyles() {
			var fontFamilyClassString = this.content.getPropertyValue('tabFontFamily') || 'ibmplexsans';
			var fontFamilySubClassString = this.content.getPropertyValue('tabFontFamilySubClass') || 'regular';
			var fontFamilyClass = TextEditorConstants.FONT_FAMILY_CHOICES.find(function (choice) {
				return fontFamilyClassString === choice.value;
			});
			var fontClass = fontFamilyClass.cssClass;
			if (fontFamilyClass.subCssClasses && fontFamilyClass.subCssClasses.length > 0) {
				var fontFamilySubClass = fontFamilyClass.subCssClasses.find(function (choice) {
					return fontFamilySubClassString === choice.value;
				});
				fontClass = fontFamilySubClass.cssClass;
			}
			var $fontFamilyDiscoveryNode = $('<div class=\'widget\'><div class=\'' + fontClass + '\'></div></div>');
			$('body').append($fontFamilyDiscoveryNode);
			var fontFamilyStyles = $fontFamilyDiscoveryNode.children().css(['font-family', 'font-style', 'font-weight']);
			$fontFamilyDiscoveryNode.remove();
			return fontFamilyStyles;
		};

		TabWidget.prototype._updateFontFamily = function _updateFontFamily() {
			this.$el.find('.ba-common-tabList').first().css(this._getFontFamilyStyles());
		};

		TabWidget.prototype._renderTabs = function _renderTabs(selectedId) {
			// widgetContent  is used with userPrimaryFGPaletteColor to enable the auto foreground colors
			// NOTE: widgetContent is also needed to allow dropdown type tabs to work in edit mode.
			this.$el.find('.tabCntr').addClass('widgetContent');
			_Tab.prototype._renderTabs.call(this, selectedId);
			this._updateFontFamily();
		};

		TabWidget.prototype._updateHeader = function _updateHeader() {
			var tabTitleElements = this.$el.find('.bx--form-item').first();
			if (!this.model.getParent() || !this.model.get('showDropDown')) {
				this.$el.removeClass('tabsHidden');
				tabTitleElements.addClass('hide');
			} else {
				this.$el.addClass('tabsHidden');
				tabTitleElements.removeClass('hide');
			}
		};

		TabWidget.prototype._updateProperties = function _updateProperties() {
			this._updateHeader();
			this.eventRouter.trigger('properties:refreshPane');
		};

		TabWidget.prototype._setTabsVisibility = function _setTabsVisibility() {
			var hideTabs = this.model.get('showDropDown') || !this.content.getPropertyValue('showTabsInPreview') && !this._dashboardState.getUiState().authoring;
			this.$el.toggleClass('tabsHidden', hideTabs);
		};

		/* Fetch the TabPane object given the TabList tab node.
   */


		TabWidget.prototype._getPanelFromNode = function _getPanelFromNode(tabNode) {
			if (tabNode) {
				// Handle child tab widget - use first() to scope to the proper tab list
				//const tabIndex = this.$el.find('.ba-common-tabList [role=tab]').index(tabNode);
				var tabIndex = this.$el.find('.ba-common-tabList').first().find('[role=tab]').index(tabNode);

				if (tabIndex > -1) {
					return this._TabsWidget.state.tabs[tabIndex];
				}
			}
			return null;
		};

		TabWidget.prototype.getSelectedSubViewId = function getSelectedSubViewId() {
			return this.selectedModel && this.selectedModel.id;
		};

		return TabWidget;
	}(Tab);

	return TabWidget;
});