'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Licensed Materials - Property of IBM
 *
 * IBM Cognos Products: BI Glass
 *
 * Copyright IBM Corp. 2021
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['dashboard-core/js/app/nls/StringResources', 'dashboard-core/js/lib/@waca/dashboard-common/dist/utils/TextEditorConstants'], function (resources, TextEditorConstants) {
	'use strict';

	var TabWidgetFontFamilyPropertyProvider = function () {
		function TabWidgetFontFamilyPropertyProvider(options) {
			_classCallCheck(this, TabWidgetFontFamilyPropertyProvider);

			var properties = options.features['Properties'];
			properties.registerProvider(this);
			this._content = options.content;
			this._dashboard = options.features['Dashboard.API'];
		}

		TabWidgetFontFamilyPropertyProvider.prototype.getLayoutModel = function getLayoutModel() {
			return this._content.getFeature('Models.internal').getLayoutModel();
		};

		TabWidgetFontFamilyPropertyProvider.prototype.getPropertyList = function getPropertyList() {
			var _this = this;

			var selectedCssClass = this.getLayoutModel().get('tabFontFamily') || 'ibmplexsans';
			var cssClass = TextEditorConstants.FONT_FAMILY_CHOICES.find(function (choice) {
				return choice.value === selectedCssClass;
			});
			var selectedCssSubClass = this.getLayoutModel().get('tabFontFamilySubClass') || 'regular';
			var cssSubClass = cssClass.subCssClasses && cssClass.subCssClasses.find(function (choice) {
				return choice.value === selectedCssSubClass;
			});
			var model = this.getLayoutModel();
			return [{
				id: 'tabFontFamily',
				editor: {
					sectionId: 'general.tabs.masterTabStyles.fontFamily.fontFamilyLine.left',
					position: 1,
					uiControl: {
						type: 'DropDown',
						ariaDescribedby: resources.get('textFontFamily'),
						options: TextEditorConstants.FONT_FAMILY_CHOICES,
						responsive: false,
						style: 'width: 156px;',
						value: selectedCssClass
					}
				},
				getPropertyValue: function getPropertyValue() {
					return model.get('tabFontFamily');
				},
				setPropertyValue: function setPropertyValue(value) {
					var oldValue = model.get('tabFontFamily');
					if (!oldValue || oldValue !== value) {
						model.set({
							'tabFontFamily': value
						}, {
							payloadData: {
								skipUndoRedo: true
							}
						});
					}
					// Hide tabFontFamilySubClass dropdown when it should not be available
					_this._dashboard.triggerDashboardEvent('properties:refreshPane');
				}
			}, {
				id: 'tabFontFamilySubClass',
				editor: {
					sectionId: 'general.tabs.masterTabStyles.fontFamily.fontFamilyLine.left',
					position: 2,
					hidden: !cssClass.subCssClasses || cssClass.subCssClasses.length === 0,
					uiControl: {
						type: 'DropDown',
						ariaDescribedby: resources.get('textFontFamily'),
						options: cssClass.subCssClasses,
						value: cssSubClass && cssSubClass.value,
						style: 'width:80px;',
						responsive: false
					}
				},
				getPropertyValue: function getPropertyValue() {
					return model.get('tabFontFamilySubClass');
				},
				setPropertyValue: function setPropertyValue(value) {
					var oldValue = model.get('tabFontFamilySubClass');
					if (!oldValue || oldValue !== value) {
						model.set({
							'tabFontFamilySubClass': value
						}, {
							payloadData: {
								skipUndoRedo: true
							}
						});
					}
				}
			}];
		};

		TabWidgetFontFamilyPropertyProvider.prototype.getPropertyLayoutList = function getPropertyLayoutList() {
			return [{
				id: 'fontFamily',
				type: 'Section',
				collapsible: false,
				label: resources.get('textFontFamily'),
				position: 1
			}, {
				id: 'fontFamilyLine',
				type: 'SingleLineLinks',
				items: [{
					id: 'left',
					align: 'left',
					items: []
				}, {
					id: 'right',
					align: 'right',
					items: []
				}]
			}];
		};

		return TabWidgetFontFamilyPropertyProvider;
	}();

	return TabWidgetFontFamilyPropertyProvider;
});