'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['original/dashboard-core/js/features/content/tabProperties/TabPropertiesProvider', 'dashboard-core/js/app/nls/StringResources', 'dashboard-core/js/dashboard/widgets/PropertyListUtil', 'underscore'], function (TabPropertiesProvider, StringResources, PropertyListUtil, _) {
	var TAB_POSITION = [{
		'value': 'top',
		'label': StringResources.get('propTop')
	}, {
		'value': 'left',
		'label': StringResources.get('propLeft')
	}, {
		'value': 'bottom',
		'label': StringResources.get('propBottom')
	}, {
		'value': 'right',
		'label': StringResources.get('propRight')
	}];

	var TAB_ICON_POSITION = [{
		'value': 'right',
		'label': StringResources.get('propRightTitle')
	}, {
		'value': 'left',
		'label': StringResources.get('propLeftTitle')
	}, {
		'value': 'top',
		'label': StringResources.get('propTopTitle')
	}, {
		'value': 'bottom',
		'label': StringResources.get('propBottomTitle')
	}];

	var TabWidgetPropertiesProvider = function (_TabPropertiesProvide) {
		_inherits(TabWidgetPropertiesProvider, _TabPropertiesProvide);

		function TabWidgetPropertiesProvider() {
			_classCallCheck(this, TabWidgetPropertiesProvider);

			return _possibleConstructorReturn(this, _TabPropertiesProvide.apply(this, arguments));
		}

		TabWidgetPropertiesProvider.prototype._getTabPropertiesList = function _getTabPropertiesList() {
			if (this._isInnerTab()) {
				return this._getInnerTabPropertiesList();
			}
			return _TabPropertiesProvide.prototype._getTabPropertiesList.call(this);
		};

		TabWidgetPropertiesProvider.prototype.getPropertyLayoutList = function getPropertyLayoutList() {
			if (this._isInnerTab()) {
				return this._getInnerTabPropertyLayoutList();
			}
			return _TabPropertiesProvide.prototype.getPropertyLayoutList.call(this);
		};

		TabWidgetPropertiesProvider.prototype._getInnerTabPropertiesList = function _getInnerTabPropertiesList() {
			var _this2 = this;

			var restoreMasterStyling = function restoreMasterStyling() {
				var transaction = _this2._dashboard.getFeature('Transaction');
				var transactionToken = transaction.startTransaction();
				_this2._content.setPropertyValue('tabTextColor', 'transparent', transactionToken);
				_this2._content.setPropertyValue('tabSelectedLineColor', 'transparent', transactionToken);
				_this2._content.setPropertyValue('tabBackgroundColor', 'transparent', transactionToken);
				transaction.endTransaction(transactionToken);
				_this2._dashboard.triggerDashboardEvent('properties:refreshPane', {
					focusSelector: '.clickCallback_left_0_tabRestoreDefaults'
				});
			};

			var sectionId = void 0,
			    sectionIndividualTabStylesId = void 0;
			if (this.isPropertiesV2Enabled) {
				sectionId = 'tabs';
				sectionIndividualTabStylesId = 'tabs.individualTabStyles';
			} else {
				sectionId = 'general.tabs';
				sectionIndividualTabStylesId = 'general.tabs.individualTabStyles';
			}

			var model = this.getLayoutModel();
			var showDropDown = model.get('showDropDown');
			var disableShowTabs = !this._allowHidingMultipleTabs && model.items.length > 1;
			var propertyList = [{
				id: 'showTabsInPreview',
				supportsUndoRedo: true,
				getPropertyValue: function getPropertyValue() {
					return !model.get('hideTab');
				},
				setPropertyValue: function setPropertyValue(value) {
					var oldValue = !model.get('hideTab');
					if (!oldValue || oldValue !== value) {
						model.set({
							'hideTab': !value
						}, {
							payloadData: { skipUndoRedo: true }
						});
					}
				},
				editor: {
					sectionId: sectionId,
					uiControl: {
						type: 'ToggleButton',
						label: StringResources.get('propShowTabsInPreview'),
						readOnly: showDropDown || disableShowTabs,
						disabled: showDropDown || disableShowTabs
					}
				}
			}, {
				id: 'showDropDown',
				getPropertyValue: function getPropertyValue() {
					return model.get('showDropDown');
				},
				setPropertyValue: function setPropertyValue(value, options) {
					var data = {};
					if (options && options.transactionId) {
						data.undoRedoTransactionId = options.transactionId;
					}
					if (options && options.transactionToken) {
						data.transactionToken = options.transactionToken;
					}
					var oldOptions = {
						payloadData: data
					};
					var oldValue = model.get('showDropDown');
					if (!oldValue || oldValue !== value) {
						model.set({
							'showDropDown': value
						}, oldOptions);
					}
				},
				editor: {
					sectionId: sectionId,
					uiControl: {
						type: 'ToggleButton',
						label: 'Show dropdown',
						readOnly: false,
						disabled: false
					}
				}
			}];

			if (!showDropDown) {
				propertyList.push({
					id: 'tabPosition',
					supportsUndoRedo: true,
					getPropertyValue: function getPropertyValue() {
						return model.get('tabPosition');
					},
					setPropertyValue: function setPropertyValue(value) {
						var oldValue = model.get('tabPosition');
						if (!oldValue || oldValue !== value) {
							model.set({
								'tabPosition': value
							}, {
								payloadData: { skipUndoRedo: true }
							});
						}
					},
					editor: {
						sectionId: sectionId,
						uiControl: {
							type: 'DropDown',
							label: StringResources.get('propTabPosition'),
							options: TAB_POSITION
						}
					}
				}, {
					id: 'tabIconPosition',
					supportsUndoRedo: true,
					getPropertyValue: function getPropertyValue() {
						return model.get('tabIconPosition');
					},
					setPropertyValue: function setPropertyValue(value) {
						var oldValue = model.get('tabIconPosition');
						if (!oldValue || oldValue !== value) {
							model.set({
								'tabIconPosition': value
							}, {
								payloadData: { skipUndoRedo: true }
							});
						}
					},
					editor: {
						sectionId: sectionId,
						uiControl: {
							type: 'DropDown',
							label: StringResources.get('propTabIconPosition'),
							options: TAB_ICON_POSITION
						}
					}
				}, {
					id: 'tabTextColor',
					supportsUndoRedo: true,
					setPropertyValue: function setPropertyValue(value, transactionToken) {
						_this2._changeTabColor('tabTextColor', value, transactionToken);
					},
					editor: {
						sectionId: 'general.tabs.masterTabStyles',
						uiControl: {
							type: 'ColorPicker',
							label: StringResources.get('propTabTextColor'),
							ariaLabel: StringResources.get('propTabTextColor'),
							paletteType: 'DashboardColorSet',
							open: false,
							showHexValue: true,
							addButton: true,
							onChange: function onChange(propertyName) {
								var propertyValueInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

								_this2._changeTabColor('tabTextColor', propertyValueInfo.name);
							}
						}
					}
				}, {
					id: 'tabSelectedLineColor',
					supportsUndoRedo: true,
					setPropertyValue: function setPropertyValue(value, transactionToken) {
						_this2._changeTabColor('tabSelectedLineColor', value, transactionToken);
					},
					editor: {
						sectionId: 'general.tabs.masterTabStyles',
						uiControl: {
							type: 'ColorPicker',
							label: StringResources.get('propTabSelectedBarColor'),
							ariaLabel: StringResources.get('propTabSelectedBarColor'),
							paletteType: 'DashboardColorSet',
							open: false,
							showHexValue: true,
							addButton: true,
							onChange: function onChange(propertyName) {
								var propertyValueInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

								_this2._changeTabColor('tabSelectedLineColor', propertyValueInfo.name);
							}
						}
					}
				}, {
					id: 'tabBackgroundColor',
					supportsUndoRedo: true,
					setPropertyValue: function setPropertyValue(value, transactionToken) {
						_this2._changeTabColor('tabBackgroundColor', value, transactionToken);
					},
					editor: {
						sectionId: 'general.tabs.masterTabStyles',
						uiControl: {
							type: 'ColorPicker',
							label: StringResources.get('propTabBackgroundColor'),
							ariaLabel: StringResources.get('propTabBackgroundColor'),
							paletteType: 'DashboardColorSet',
							open: false,
							showHexValue: true,
							addButton: true,
							onChange: function onChange(propertyName) {
								var propertyValueInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

								_this2._changeTabColor('tabBackgroundColor', propertyValueInfo.name);
							}
						}
					}
				}, {
					id: 'tabRestoreDefaults',
					editor: {
						sectionId: 'general.tabs.masterTabStyles',
						uiControl: {
							type: 'SingleLineLinks',
							items: [{
								align: 'left',
								items: [{
									type: 'icon',
									class: 'iconTextLink',
									svgIcon: this._features['Dashboard.Icons'].getIcon('restore').id,
									iconTooltip: StringResources.get('propTabRestoreDefaults'),
									clickCallback: restoreMasterStyling
								}]
							}, {
								align: 'right',
								items: [{
									type: 'text',
									class: 'iconTextLink',
									value: StringResources.get('propTabRestoreDefaults'),
									ariaLabel: StringResources.get('propTabRestoreDefaults'),
									clickCallback: restoreMasterStyling
								}]
							}]
						}
					}
				}, {
					id: 'allowHidingMultipleTabs',
					getPropertyValue: function getPropertyValue() {
						return _this2._allowHidingMultipleTabs;
					},
					setPropertyValue: function setPropertyValue() {
						throw new Error('not supported');
					}
				});
				propertyList.push.apply(propertyList, this._getIndividualTabStylesLinks(sectionIndividualTabStylesId));
			}

			var leftProperty = {
				id: 'left',
				getPropertyValue: function getPropertyValue() {
					if (model.style) {
						return model.style.left;
					}
				},
				setPropertyValue: function setPropertyValue(value, options) {
					_this2._updateStyleValueInModel('left', value, options);
				},
				editor: {
					sectionId: 'general.layout.widgetPosition.moveWidgetSplit.left',
					uiControl: {
						type: 'InputLabel',
						label: StringResources.get('propPositionXAxis'),
						multiline: true,
						handleReturnKey: true,
						decimalPlaces: 2,
						onChange: this._onChangePercentOrPixelProperty.bind(this, 'width')
					}
				},
				onPropertyChange: {
					refresh: {
						propertiesPane: true
					}
				}
			};

			var topProperty = {
				id: 'top',
				getPropertyValue: function getPropertyValue() {
					if (model.style) {
						return model.style.top;
					}
				},
				setPropertyValue: function setPropertyValue(value, options) {
					_this2._updateStyleValueInModel('top', value, options);
				},
				editor: {
					sectionId: 'general.layout.widgetPosition.moveWidgetSplit.right',
					uiControl: {
						type: 'InputLabel',
						label: StringResources.get('propPositionYAxis'),
						multiline: true,
						handleReturnKey: true,
						decimalPlaces: 2,
						onChange: this._onChangePercentOrPixelProperty.bind(this, 'height')
					}
				},
				onPropertyChange: {
					refresh: {
						propertiesPane: true
					}
				}
			};

			var widthProperty = {
				id: 'width',
				getPropertyValue: function getPropertyValue() {
					if (model.style) {
						return model.style.width;
					}
				},
				setPropertyValue: function setPropertyValue(value, options) {
					_this2._updateStyleValueInModel('width', value, options);
				},
				editor: {
					sectionId: 'general.layout.resizeWidget.resizeWidgetSplit.left',
					uiControl: {
						type: 'InputLabel',
						label: StringResources.get('propResizeWidth'),
						multiline: true,
						handleReturnKey: true,
						decimalPlaces: 2,
						onChange: this._onChangePercentOrPixelProperty.bind(this, 'left')
					}
				},
				onPropertyChange: {
					refresh: {
						propertiesPane: true
					}
				}
			};

			var heightProperty = {
				id: 'height',
				getPropertyValue: function getPropertyValue() {
					if (model.style) {
						return model.style.height;
					}
				},
				setPropertyValue: function setPropertyValue(value, options) {
					_this2._updateStyleValueInModel('height', value, options);
				},
				editor: {
					sectionId: 'general.layout.resizeWidget.resizeWidgetSplit.right',
					uiControl: {
						type: 'InputLabel',
						label: StringResources.get('propResizeHeight'),
						multiline: true,
						handleReturnKey: true,
						decimalPlaces: 2,
						onChange: this._onChangePercentOrPixelProperty.bind(this, 'top')
					}
				},
				onPropertyChange: {
					refresh: {
						propertiesPane: true
					}
				}
			};

			propertyList.push(leftProperty, topProperty, widthProperty, heightProperty);
			return propertyList;
		};

		TabWidgetPropertiesProvider.prototype._getInnerTabPropertyLayoutList = function _getInnerTabPropertyLayoutList() {
			var propertyLayoutList = [{
				id: 'general',
				type: 'Group',
				label: StringResources.get('tabName_general')
			}, {
				'id': 'banner',
				'type': 'Banner',
				'value': 'Tab widget properties',
				'editable': false
			}, {
				id: 'tabs',
				type: 'Section',
				label: StringResources.get('sectionName_tabs'),
				position: 2
			}, {
				id: 'masterTabStyles',
				type: 'Section',
				collapsible: false,
				label: StringResources.get('propMasterTabStylesLabel')
			}, {
				id: 'individualTabStyles',
				type: 'Section',
				collapsible: false,
				label: StringResources.get('propIndividualTabStylesLabel')
			}, {
				id: 'layout',
				type: 'Section',
				open: true,
				label: StringResources.get('sectionName_layout'),
				position: 3
			}, {
				id: 'widgetPosition',
				type: 'Section',
				collapsible: false,
				label: StringResources.get('propPositionLabel'),
				position: 2
			}, {
				id: 'moveWidgetSplit',
				type: 'Split',
				items: [{
					id: 'left',
					align: 'left',
					items: []
				}, {
					id: 'right',
					align: 'right',
					items: []
				}]
			}, {
				id: 'resizeWidget',
				type: 'Section',
				collapsible: false,
				label: StringResources.get('propResizeLabel'),
				position: 3
			}, {
				id: 'resizeWidgetSplit',
				type: 'Split',
				items: [{
					id: 'left',
					align: 'left',
					items: []
				}, {
					id: 'right',
					align: 'right',
					items: []
				}]
			}];

			return propertyLayoutList;
		};

		TabWidgetPropertiesProvider.prototype._getIndividualTabStylesLinks = function _getIndividualTabStylesLinks(sectionIndividualTabStylesId) {
			var _this3 = this;

			if (this._isInnerTab()) {
				var contentPages = this._content.findContent({ type: 'page' });
				return this._getIndividualTabStylesLinksForPages(contentPages, sectionIndividualTabStylesId);
			}
			var canvasPages = this._getCanvas().findContent({ type: 'page' });
			var topLevelPages = _.filter(canvasPages, function (page) {
				return !_this3._isInnerPage(page);
			});
			return this._getIndividualTabStylesLinksForPages(topLevelPages, sectionIndividualTabStylesId);
		};

		TabWidgetPropertiesProvider.prototype._isInnerPage = function _isInnerPage(page) {
			var container = page;
			var isInnerPage = false;
			while (!isInnerPage && container) {
				container = container.getContainer();
				isInnerPage = container && container.getType() !== 'tab'; // inner tab container type is 'genericPage'
			}
			return isInnerPage;
		};

		TabWidgetPropertiesProvider.prototype._getIndividualTabStylesLinksForPages = function _getIndividualTabStylesLinksForPages(pages) {
			var _this4 = this;

			return pages.map(function (page, index) {
				var clickCallback = function clickCallback() {
					_this4._individualTabSlideOut(page);
				};

				var layoutModel = _this4.getLayoutModel(page.getId());
				var tabIcon = page.getPropertyValue('tabIcon');

				if (!tabIcon) {
					// This case is hack for now. We are not loading the features for non visible tabs which cause the property 
					// provider to to register the proper getPropertyValue.
					// TODO - The real fix is to move the property to proper place and stop using a cutom getPropertyValue
					var icon = layoutModel.get('tabIcon');
					if (icon && icon !== 'transparent') {
						tabIcon = _this4._features['Dashboard.Icons'].getIcon(icon).id;
					} else {
						tabIcon = 'transparent';
					}
				}

				// Defaulting to the layoutModel is hack for now. We are not loading the features for non visible tabs which cause the property 
				// provider to to register the proper getPropertyValue.
				// TODO - The real fix is to move the property to proper place and stop using a cutom getPropertyValue
				var title = page.getPropertyValue('title') || layoutModel.get('title');

				return {
					id: 'tabIndividualStylingLink' + index,
					editor: {
						sectionId: 'general.tabs.individualTabStyles',
						uiControl: {
							type: 'SingleLineLinks',
							clickCallback: clickCallback,
							items: [{
								align: 'left',
								items: [{
									type: 'icon',
									svgIcon: tabIcon !== 'transparent' ? tabIcon : null,
									iconTooltip: StringResources.get('propIndividualTabStylesAriaLabel')
								}]
							}, {
								align: 'right',
								items: [{
									type: 'text',
									value: title,
									showRightArrow: true
								}]
							}]
						}
					}
				};
			});
		};

		TabWidgetPropertiesProvider.prototype._isInnerTab = function _isInnerTab() {
			return this._content.getType() === 'tab' && this._content.getContainer();
		};

		TabWidgetPropertiesProvider.prototype._updateStyleValueInModel = function _updateStyleValueInModel(styleName, value) {
			var styleValue = value;
			var model = this.getLayoutModel();
			var newStyle = _.extend({}, model.style);
			newStyle[styleName] = styleValue;

			// Todo: we can stop using the topLayoutModel once the undo/redo uses
			// the API events, but for now using the model directly doesn't add to the stack
			// if the widget is not on the same tab that it was initally added on.
			model.getTopLayoutModel().updateModel({
				updateArray: [{
					id: model.id,
					style: newStyle
				}]
			}, this, {
				skipUndoRedo: true
			});

			return styleValue;
		};

		TabWidgetPropertiesProvider.prototype._onChangePercentOrPixelProperty = function _onChangePercentOrPixelProperty(constrainedBy, propertyName, propertyValue, options) {
			var units = this.getUnit();
			var minimum = 25;
			if (minimum && units === '%') {
				// get the minimum value to percent
				var domFeature = this._content.getContainer().getFeature('ContentViewDOM');
				if (domFeature) {
					var node = domFeature.getNode();
					var size = propertyName === 'height' ? node.clientHeight : node.clientWidth;
					minimum = 100 * minimum / size;
				} else {
					// without a dom we can't figure out the minimum percent.. so we use 0
					minimum = 0;
				}
			}

			var numericValue = this._getNumericValueWithConstraint(constrainedBy, propertyValue, minimum);

			var formattedValue = PropertyListUtil.getPropertyDisplayString(numericValue, units);

			this._content.setPropertyValue(propertyName, formattedValue, options);
		};

		TabWidgetPropertiesProvider.prototype.getUnit = function getUnit() {
			return this._getCanvas().getPropertyValue('layoutPositioning') === 'absolute' ? 'px' : '%';
		};

		TabWidgetPropertiesProvider.prototype._getMaxValueForConstraint = function _getMaxValueForConstraint(constrainedBy) {
			if (this._getCanvas().getPropertyValue('layoutPositioning') === 'relative') {
				return 100;
			} else {
				if (constrainedBy === 'width' || constrainedBy === 'left') {
					return parseFloat(this._getCanvas().getPropertyValue('pageSizeWidth'));
				} else {
					return parseFloat(this._getCanvas().getPropertyValue('pageSizeHeight'));
				}
			}
		};

		TabWidgetPropertiesProvider.prototype._getNumericValueWithConstraint = function _getNumericValueWithConstraint(constrainedBy, propertyValue, minimum) {
			var numericValue = parseFloat(propertyValue);
			if (isNaN(numericValue)) {
				numericValue = 0;
			}
			var constraint = this.content.getPropertyValue(constrainedBy) || 0;
			var maxValue = this._getMaxValueForConstraint(constrainedBy);
			numericValue = Math.max(Math.min(maxValue, numericValue), minimum || 0);
			constraint = parseFloat(constraint);
			if (constraint + numericValue > maxValue) {
				numericValue = maxValue - constraint;
			}

			return numericValue;
		};

		return TabWidgetPropertiesProvider;
	}(TabPropertiesProvider);

	return TabWidgetPropertiesProvider;
});